'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setMetadata = exports.download = exports.fetchFromYoutube = exports.fetchFromiTunes = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _http = require('http');

var _http2 = _interopRequireDefault(_http);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _chalk = require('chalk');

var _chalk2 = _interopRequireDefault(_chalk);

var _nodeId = require('node-id3');

var _nodeId2 = _interopRequireDefault(_nodeId);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _untildify = require('untildify');

var _untildify2 = _interopRequireDefault(_untildify);

var _youtubeNode = require('youtube-node');

var _youtubeNode2 = _interopRequireDefault(_youtubeNode);

var _itunesInfo = require('itunes-info');

var _youtubeDlStatus = require('youtube-dl-status');

var _youtubeDlStatus2 = _interopRequireDefault(_youtubeDlStatus);

var _emitter = require('./emitter');

var _emitter2 = _interopRequireDefault(_emitter);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Converts a time string, provided by Youtube, to the total amount of millis.
 * For example - PT1M20S => 80000.
 * @param {string} str The string provided by Youtube.
 * @returns {number} The total amount of millis.
 */
var timeStrToMillis = function timeStrToMillis(str) {
  var units = { S: 1000, M: 1000 * 60, H: 1000 * 60 * 60, D: 1000 * 60 * 60 * 24 };
  var content = str.replace('PT', '');
  var numbers = content.split(/\D/g).filter(function (e) {
    return !!e;
  });
  var symbols = content.match(/\D/g).filter(function (e) {
    return !!e;
  });
  var result = numbers.reduce(function (a, b, i) {
    return a + b * units[symbols[i]];
  }, 0);
  return result;
};

/**
 * Returns a promise which fetches the details of the given Youtube result with the given
 * Youtube instance. The api should already be set.
 * @param {Object} youtube The Youtube instance.
 * @param {Object} element The Youtube result.
 * @returns {Promise<Object>} The details of the Youtube result.
 */
var expandYoutube = function expandYoutube(youtube, element) {
  return new _bluebird2.default(function (resolve, reject) {
    youtube.getById(element.id.videoId, function (error, result) {
      if (error) {
        return reject(error);
      }
      if (result.items.length < 1 || !result.items[0]) {
        return resolve(undefined);
      }
      var video = result.items[0];
      video.link = 'https://www.youtube.com/watch?v=' + video.id;
      video.durationMillis = timeStrToMillis(video.contentDetails.duration);
      return resolve(video);
    });
  });
};

/**
 * Fetch the data from iTunes for the given query. The query should be provided
 * as a 'query' option. An optional emitter can be provided too. The path of the
 * track will be set to the directory as provided as 'directory'. Everything
 * is optional!
 * @param {?string} options.query The query to send to the iTunes store.
 * @param {?Emitter} options.emitter Optional EventEmitter.
 * @param {?string} options.directory The directory of the track to download to.
 * @return {Promise<Object>} Promise with track details, merged with the options!
 */
var fetchFromiTunes = exports.fetchFromiTunes = function fetchFromiTunes(options) {
  var emitter = options.emitter || new _emitter2.default();
  emitter.updateState('Connecting to iTunes...');
  return _itunesInfo.iTunes.fetch(options.query || '').then(function (data) {
    emitter.updateState('Fetched iTunes data...');
    emitter.log('Received a total of ' + data.results.length + ' results.');
    var tracks = data.results.filter(function (res) {
      return res.kind === 'song';
    });
    if (tracks.length < 1) {
      emitter.warning('Could not find this track on iTunes, will not set the id3 tags.');
      return options;
    }
    emitter.log(tracks.length + ' of them are songs.');
    tracks[0].artworkUrl512 = tracks[0].artworkUrl100.replace('100x100', '512x512');
    tracks[0].filename = tracks[0].artistName + ' - ' + tracks[0].trackCensoredName;
    tracks[0].path = {
      file: _path2.default.resolve((0, _untildify2.default)(options.directory) || process.cwd(), tracks[0].filename + '.mp3'),
      image: _path2.default.resolve((0, _untildify2.default)(options.directory) || process.cwd(), tracks[0].filename + '.jpg')
    };
    emitter.log('Will take this track to proceed:\n' + _chalk2.default.cyan(JSON.stringify(tracks[0], null, 4)));
    return _extends({}, options, tracks[0]);
  });
};

/**
 * Fetch the data from Youtube with given filename or query. Filename will be used if
 * found, as this means that it matches an iTunes result (result of fetchFromiTunes).
 * If this one is not provided, the query option will be used.
 * @param {?string} options.filename The filename to query for on Youtube (from iTunes).
 * @param {?string} options.query The query to search for (if no filename provided).
 * @param {?string} options.token The token for the Youtube API.
 * @param {?Emitter} options.emitter Optional EventEmitter.
 * @param {?number} options.results The total amount of results to check (otherwise 50).
 * @param {?number} options.trackTimeMillis The millis of the track from iTunes (to match).
 * @return {Promise<Object>} Promise with video details, merged with the options!
 */
var fetchFromYoutube = exports.fetchFromYoutube = function fetchFromYoutube(options) {
  return new _bluebird2.default(function (resolve, reject) {
    if (options.link) {
      return resolve(options);
    }
    var emitter = options.emitter || new _emitter2.default();
    emitter.updateState('Finding a Youtube match...');
    emitter.log('Using "' + (options.token || '') + '" as token for Youtube.');
    var youtube = new _youtubeNode2.default();
    youtube.setKey(options.token);
    var query = options.filename || options.query || '';
    emitter.log('Using Youtube query: ' + query);
    return youtube.search(query, options.results || 50, function (error, results) {
      if (error) {
        return reject(new Error('Bad Youtube request, do you use a valid token?'));
      }
      emitter.log('Found a total of ' + results.items.length + ' tracks on Youtube.');
      if (results.items.length === 0) {
        return reject(new Error('No YouTube results found!'));
      }
      return _bluebird2.default.map(results.items, function (res) {
        return expandYoutube(youtube, res);
      }, { concurrency: 50 }).then(function (videos) {
        var list = videos.filter(function (v) {
          return !!v;
        });
        emitter.log('Found a total of ' + list.length + ' valid videos on Youtube.');
        if (list.length === 0) {
          if (options.filename) {
            return reject(new Error('The first result of iTunes is ' + options.filename + '.\nHowever, there are no youtube results for this title.\nIf this was not your desired result, please provide a more specific query.'));
          }
        }
        if (!options.filename) {
          emitter.log('Will use this Youtube result:\n' + _chalk2.default.cyan(JSON.stringify(list[0], null, 4)));
          return resolve(_extends({}, options, list[0]));
        }
        list.forEach(function (res) {
          res.timeDifference = Math.abs(res.durationMillis - (options.trackTimeMillis || 0));
        });
        list.sort(function (a, b) {
          return a.timeDifference - b.timeDifference;
        });
        emitter.log('Will use this Youtube result:\n' + _chalk2.default.cyan(JSON.stringify(list[0], null, 4)));
        return resolve(_extends({}, options, list[0]));
      });
    });
  });
};

/**
 * Downloads the track from youtube with the provided options.
 * @param {string} options.link The directory of the track to download to.
 * @param {?Emitter} options.emitter Optional emitter to get updates.
 * @param {?string} options.filename The filename to download to (from iTunes).
 * @param {?string} options.query The query (as backup for filename in case of no iTunes result).
 * @param {?string} options.directory The directory of the track to download to.
 * @return {Promise<Object>} Promise when download is ready.
 */
var download = exports.download = function download(options) {
  return new _bluebird2.default(function (resolve) {
    if (!options.link) {
      throw new Error('FATAL: Lost Youtube link!');
    }
    var emitter = options.emitter || new _emitter2.default();
    emitter.updateState('Starting download...');
    var filename = options.filename || options.query || 'track';
    var conf = ['--extract-audio', '--audio-quality=0', '--format=bestaudio', '--audio-format=mp3', '--output=' + _path2.default.resolve((0, _untildify2.default)(options.directory) || process.cwd(), filename + '.%(ext)s')];
    var downloadProcess = _youtubeDlStatus2.default.exec(options.link, conf, function () {
      resolve(options);
    });
    downloadProcess.stdout.on('data', function (data) {
      var re = /[0-9]+((\.[0-9]{1}){0,1})%/i;
      var matches = data.match(re);
      if (matches && matches.length && matches.length > 0) {
        emitter.updateState(matches[0] !== '100%' ? 'Downloading track - ' + matches[0] : 'Converting...');
      }
    });
  });
};

/**
 * Will write the tags to the provided mp3 file. Requires all options from iTunes result!
 * @param {?Emitter} options.emitter Optional emitter to get updates.
 */
var setMetadata = exports.setMetadata = function setMetadata(options) {
  return new _bluebird2.default(function (resolve, reject) {
    var emitter = options.emitter || new _emitter2.default();
    emitter.updateState('Writing metadata...');
    if (!options.filename) {
      emitter.success('Finished without metadata.');return resolve();
    }
    emitter.updateState('Fetching artwork...');
    return _http2.default.get(options.artworkUrl512, function (response) {
      if (response.statusCode === 200) {
        emitter.updateState('Received artwork.');
        response.pipe(_fs2.default.createWriteStream(options.path.image)).on('finish', function () {
          emitter.updateState('Writing tags...');
          _nodeId2.default.removeTags(options.path.file);
          var success = _nodeId2.default.write({
            title: options.trackCensoredName,
            artist: options.artistName,
            album: options.collectionCensoredName,
            image: options.path.image,
            trackNumber: options.trackNumber,
            genre: options.primaryGenreName,
            date: options.releaseDate,
            year: options.releaseDate.substring(0, 4)
          }, options.path.file);
          if (!success) {
            return reject(new Error('Unable to write metadata!'));
          }
          emitter.success('Finished writing metadata!');
          return _fs2.default.unlink(options.path.image, resolve);
        });
      } else {
        emitter.success('Issue fetching image, finished without metadata.');resolve();
      }
    });
  });
};